@@ -54,11 +54,18 @@ from atlas_commons.app_utils import (
     common_atlas_options,
     log_args,
     set_verbose,
+    verbose_option,
 )
+from atlas_commons.typing import FloatArray
 from voxcell import RegionMap, VoxelData  # type: ignore
 
 from atlas_densities.app.utils import AD_PATH, DATA_PATH
-from atlas_densities.densities import excitatory_inhibitory_splitting
+from atlas_densities.densities import (
+    excitatory_inhibitory_splitting,
+    inhibitory_neuron_densities_optimization,
+    refined_inhibitory_neuron_densities,
+    utils,
+)
 from atlas_densities.densities.cell_counts import (
     extract_inhibitory_neurons_dataframe,
     glia_cell_counts,
@@ -71,18 +78,11 @@ from atlas_densities.densities.excel_reader import (
 )
 from atlas_densities.densities.fitting import linear_fitting
 from atlas_densities.densities.glia_densities import compute_glia_densities
-from atlas_densities.densities.inhibitory_neuron_densities_optimization import (
-    create_inhibitory_neuron_densities as linprog,
-)
 from atlas_densities.densities.inhibitory_neuron_density import compute_inhibitory_neuron_density
 from atlas_densities.densities.measurement_to_density import (
     measurement_to_average_density,
     remove_non_density_measurements,
 )
-from atlas_densities.densities.refined_inhibitory_neuron_densities import (
-    create_inhibitory_neuron_densities as keep_proportions,
-)
-from atlas_densities.densities.utils import zero_negative_values
 from atlas_densities.exceptions import AtlasDensitiesError
 
 EXCITATORY_SPLIT_CORTEX_ALL_TO_EXC_MTYPES = (
@@ -95,13 +95,49 @@ MARKERS_README_REL_PATH = (DATA_PATH / "markers" / "README.rst").relative_to(AD_
 LINPROG_PATH = "doc/source/bbpp82_628_linprog.pdf"
 
 ALGORITHMS: Dict[str, Callable] = {
-    "keep-proportions": keep_proportions,
-    "linprog": linprog,
+    "keep-proportions": refined_inhibitory_neuron_densities.create_inhibitory_neuron_densities,
+    "linprog": inhibitory_neuron_densities_optimization.create_inhibitory_neuron_densities,
 }
 
 L = logging.getLogger(__name__)
 
 
+def _zero_negative_values(array: FloatArray) -> None:
+    """
+    Zero negative values resulting from round-off errors.
+
+    Modifies `array` in place.
+
+    Args:
+        array: float numpy array. We expect most of the `array` values to be non-negative.
+            Negative values should be negligible when compared to positive values.
+
+    Raises:
+        AtlasDensitiesError if the absolute value of the sum of all negative values exceeds
+            1 percent of the sum of all positive values, or if the smallest negative value is
+            not negligible wrt to the mean of the non-negative values.
+    """
+    negative_mask = array < 0.0
+    if np.count_nonzero(negative_mask) == 0:
+        return
+
+    non_negative_mask = np.invert(negative_mask)
+
+    if np.abs(np.sum(array[negative_mask])) / np.sum(array[non_negative_mask]) > 0.01:
+        raise AtlasDensitiesError(
+            "The absolute value of the sum of all negative values exceeds"
+            " 1 percent of the sum of all positive values"
+        )
+    ratio = np.abs(np.min(array[negative_mask])) / np.mean(array[non_negative_mask])
+    if not np.isclose(ratio, 0.0, atol=1e-08):
+        raise AtlasDensitiesError(
+            "The smallest negative value is not negligible wrt to "
+            "the mean of all non-negative values."
+        )
+
+    array[negative_mask] = 0.0
+
+
 def _get_voxel_volume_in_mm3(voxel_data: "VoxelData") -> float:
     """
     Returns the voxel volume of `voxel_data` in mm^3.
@@ -119,7 +155,7 @@ def _get_voxel_volume_in_mm3(voxel_data: "VoxelData") -> float:
 
 
 @click.group()
-@click.option("-v", "--verbose", count=True)
+@verbose_option
 def app(verbose):
     """Run the cell densities CLI"""
     set_verbose(L, verbose)
@@ -140,8 +176,15 @@ def app(verbose):
     help="Path where to write the output cell density nrrd file."
     "A voxel value is a number of cells per mm^3",
 )
+@click.option(
+    "--group-ids-config-path",
+    type=EXISTING_FILE_PATH,
+    default=utils.GROUP_IDS_PATH,
+    help="Path to density groups ids config",
+    show_default=True,
+)
 @log_args(L)
-def cell_density(annotation_path, hierarchy_path, nissl_path, output_path):
+def cell_density(annotation_path, hierarchy_path, nissl_path, output_path, group_ids_config_path):
     """Compute and save the overall mouse brain cell density.
 
     The input Nissl stain volume of AIBS is turned into an actual density field complying with
@@ -155,7 +198,7 @@ def cell_density(annotation_path, hierarchy_path, nissl_path, output_path):
 
     \b
     - the Nissl stain intensity, which is supposed to represent the overall cell density, up to
-    to region-dependent constant scaling factors.
+    region-dependent constant scaling factors.
     - cell counts from the scientific literature, which are used to determine a local
     linear dependency factor for each region where a cell count is available.
     - the optional soma radii, used to operate a correction.
@@ -168,12 +211,14 @@ def cell_density(annotation_path, hierarchy_path, nissl_path, output_path):
     assert_properties([annotation, nissl])
 
     region_map = RegionMap.load_json(hierarchy_path)
+    group_ids_config = utils.load_json(group_ids_config_path)
 
     overall_cell_density = compute_cell_density(
         region_map,
         annotation.raw,
         _get_voxel_volume_in_mm3(annotation),
         nissl.raw,
+        group_ids_config=group_ids_config,
     )
     nissl.with_data(overall_cell_density).save_nrrd(output_path)
 
@@ -225,6 +270,13 @@ def cell_density(annotation_path, hierarchy_path, nissl_path, output_path):
     help="Path to the directory where to write the output cell density nrrd files."
     " It will be created if it doesn't exist already.",
 )
+@click.option(
+    "--group-ids-config-path",
+    type=EXISTING_FILE_PATH,
+    default=utils.GROUP_IDS_PATH,
+    help="Path to density groups ids config",
+    show_default=True,
+)
 @log_args(L)
 def glia_cell_densities(
     annotation_path,
@@ -236,6 +288,7 @@ def glia_cell_densities(
     microglia_density_path,
     glia_proportions_path,
     output_dir,
+    group_ids_config_path,
 ):  # pylint: disable=too-many-arguments, too-many-locals
     """Compute and save the glia cell densities.
 
@@ -295,20 +348,20 @@ def glia_cell_densities(
         "microglia": VoxelData.load_nrrd(microglia_density_path),
     }
 
-    atlases = list(glia_densities.values())
-    atlases += [annotation, overall_cell_density]
+    atlases = list(glia_densities.values()) + [annotation, overall_cell_density]
     L.info("Checking input files consistency ...")
     assert_properties(atlases)
 
     L.info("Loading hierarchy ...")
     region_map = RegionMap.load_json(hierarchy_path)
-    with open(glia_proportions_path, "r", encoding="utf-8") as file_:
-        glia_proportions = json.load(file_)
+    glia_proportions = utils.load_json(glia_proportions_path)
 
     glia_densities = {
-        glia_cell_type: voxel_data.raw for (glia_cell_type, voxel_data) in glia_densities.items()
+        glia_cell_type: voxel_data.raw for glia_cell_type, voxel_data in glia_densities.items()
     }
 
+    group_ids_config = utils.load_json(group_ids_config_path)
+
     L.info("Compute volumetric glia densities: started")
     glia_densities = compute_glia_densities(
         region_map,
@@ -319,6 +372,7 @@ def glia_cell_densities(
         overall_cell_density.raw,
         glia_proportions,
         copy=False,
+        group_ids_config=group_ids_config,
     )
 
     if not Path(output_dir).exists():
@@ -326,7 +380,7 @@ def glia_cell_densities(
 
     L.info("Saving overall neuron density to file %s", str(Path(output_dir, "neuron_density.nrrd")))
     neuron_density = overall_cell_density.raw - glia_densities["glia"]
-    zero_negative_values(neuron_density)
+    _zero_negative_values(neuron_density)
     annotation.with_data(np.asarray(neuron_density, dtype=float)).save_nrrd(
         str(Path(output_dir, "neuron_density.nrrd"))
     )
@@ -380,6 +434,13 @@ def glia_cell_densities(
     help="Path to the directory where to write the output cell density nrrd files."
     " It will be created if it doesn't exist already.",
 )
+@click.option(
+    "--group-ids-config-path",
+    type=EXISTING_FILE_PATH,
+    default=utils.GROUP_IDS_PATH,
+    help="Path to density groups ids config",
+    show_default=True,
+)
 @log_args(L)
 def inhibitory_and_excitatory_neuron_densities(
     annotation_path,
@@ -389,6 +450,7 @@ def inhibitory_and_excitatory_neuron_densities(
     neuron_density_path,
     inhibitory_neuron_counts_path,
     output_dir,
+    group_ids_config_path,
 ):  # pylint: disable=too-many-arguments
     """Compute and save the inhibitory and excitatory neuron densities.
 
@@ -417,7 +479,7 @@ def inhibitory_and_excitatory_neuron_densities(
     Note: optimization is not fully implemented and the current process only returns a
     feasible point.
 
-    The ouput densities are saved in the specified output directory under the following
+    The output densities are saved in the specified output directory under the following
     names:
 
     \b
@@ -434,6 +496,7 @@ def inhibitory_and_excitatory_neuron_densities(
 
     region_map = RegionMap.load_json(hierarchy_path)
     inhibitory_df = extract_inhibitory_neurons_dataframe(inhibitory_neuron_counts_path)
+    group_ids_config = utils.load_json(group_ids_config_path)
     inhibitory_neuron_density = compute_inhibitory_neuron_density(
         region_map,
         annotation.raw,
@@ -442,6 +505,7 @@ def inhibitory_and_excitatory_neuron_densities(
         VoxelData.load_nrrd(nrn1_path).raw,
         neuron_density.raw,
         inhibitory_data=inhibitory_data(inhibitory_df),
+        group_ids_config=group_ids_config,
     )
 
     if not Path(output_dir).exists():
@@ -451,7 +515,7 @@ def inhibitory_and_excitatory_neuron_densities(
         str(Path(output_dir, "inhibitory_neuron_density.nrrd"))
     )
     excitatory_neuron_density = neuron_density.raw - inhibitory_neuron_density
-    zero_negative_values(excitatory_neuron_density)
+    _zero_negative_values(excitatory_neuron_density)
     annotation.with_data(np.asarray(excitatory_neuron_density, dtype=float)).save_nrrd(
         str(Path(output_dir, "excitatory_neuron_density.nrrd"))
     )
@@ -490,7 +554,7 @@ def compile_measurements(
     Sexual Dimorphism' by Kim et al., 2017.
     https://ars.els-cdn.com/content/image/1-s2.0-S0092867417310693-mmc3.xlsx
     - `atlas_densities/app/data/measurements/gaba_papers.xlsx`, a compilation of measurements
-    from the scientifc literature made by Rodarie Dimitri (BBP).
+    from the scientific literature made by Rodarie Dimitri (BBP).
 
     This command extracts measurements from the above two files and gathers them into a unique
     CSV file with the following columns:
@@ -531,7 +595,7 @@ def compile_measurements(
     See `atlas_densities/densities/excel_reader.py` for more information.
 
     Note: This function should be deprecated once its output has been stored permanently as the
-    the unique source of density-related measurements for the AIBS mouse brain. New measurements
+    unique source of density-related measurements for the AIBS mouse brain. New measurements
     should be added to the stored file (Nexus).
     """
 
@@ -557,6 +621,12 @@ def compile_measurements(
 
 @app.command()
 @common_atlas_options
+@click.option(
+    "--region-name",
+    type=str,
+    default="root",
+    help="Name of the root region in the hierarchy",
+)
 @click.option(
     "--cell-density-path",
     type=EXISTING_FILE_PATH,
@@ -590,6 +660,7 @@ def compile_measurements(
 def measurements_to_average_densities(
     annotation_path,
     hierarchy_path,
+    region_name,
     cell_density_path,
     neuron_density_path,
     measurements_path,
@@ -602,6 +673,10 @@ def measurements_to_average_densities(
     `neuron_density_path`) are used to compute average cell densities in every AIBS region where
     sufficient information is available.
 
+    Measurements from regions which are not in the provided brain region hierarchy or not in the
+    provided annotation volume will be ignored. A warning with all ignored lines from the
+    measurements file will be displayed.
+
     The different cell types (e.g., PV+, SST+, VIP+ or overall inhibitory neurons) and
     brain regions under consideration are prescribed by the input measurements.
 
@@ -619,7 +694,7 @@ def measurements_to_average_densities(
     If several combinations of measurements yield several average cell densities for the same
     region, then the output data frame records the average of these measurements.
 
-    The ouput average cell densities are saved in under the CSV format as a dataframe with the same
+    The output average cell densities are saved in under the CSV format as a dataframe with the same
     columns as the input data frame specified via `--measurements-path`.
     See :mod:`atlas_densities.app.densities.compile_measurements`.
 
@@ -645,6 +720,7 @@ def measurements_to_average_densities(
     region_map = RegionMap.load_json(hierarchy_path)
     L.info("Loading measurements ...")
     measurements_df = pd.read_csv(measurements_path)
+
     L.info("Measurement to average density: started")
     average_cell_densities_df = measurement_to_average_density(
         region_map,
@@ -654,6 +730,7 @@ def measurements_to_average_densities(
         overall_cell_density.raw,
         neuron_density.raw,
         measurements_df,
+        region_name,
     )
 
     remove_non_density_measurements(average_cell_densities_df)
@@ -667,6 +744,12 @@ def measurements_to_average_densities(
 
 @app.command()
 @common_atlas_options
+@click.option(
+    "--region-name",
+    type=str,
+    default="root",
+    help="Name of the root region in the hierarchy",
+)
 @click.option(
     "--neuron-density-path",
     type=EXISTING_FILE_PATH,
@@ -718,16 +801,33 @@ def measurements_to_average_densities(
     help="Path to the json file containing the fitting coefficients and standard deviations"
     "for each region group and each cell type.",
 )
+@click.option(
+    "--group-ids-config-path",
+    type=EXISTING_FILE_PATH,
+    default=utils.GROUP_IDS_PATH,
+    help="Path to density groups ids config",
+    show_default=True,
+)
+@click.option(
+    "--min-data-points",
+    type=int,
+    default=1,
+    help="minimum number of datapoints required for running the linear regression.",
+    show_default=True,
+)
 @log_args(L)
 def fit_average_densities(
     hierarchy_path,
     annotation_path,
+    region_name,
     neuron_density_path,
     gene_config_path,
     average_densities_path,
     homogenous_regions_path,
     fitted_densities_output_path,
     fitting_maps_output_path,
+    group_ids_config_path,
+    min_data_points,
 ):  # pylint: disable=too-many-arguments, too-many-locals
     """
     Estimate average cell densities of brain regions in `hierarchy_path` for the cell types
@@ -743,8 +843,12 @@ def fit_average_densities(
     `neuron_density_path` is used to compute the average density of inhibitory neurons (a.k.a
     gad67+) in every homogenous region of type "inhibitory".
 
+    Regions from the literature values and homogenous regions which are not in the provided brain
+    region hierarchy or not in the provided annotation volume will be ignored. A warning with all
+    ignored lines from the measurements file will be displayed.
+
     Our linear fitting of density values relies on the assumption that the average cell density
-    (number of cells per mm^3) of a cell type T in a brain region R depends linearily on the
+    (number of cells per mm^3) of a cell type T in a brain region R depends linearly on the
     average intensity of a gene marker of T. The conversion factor is a constant which depends only
     on T and on which of the following three groups R belongs to:
 
@@ -777,6 +881,7 @@ def fit_average_densities(
     - some regions can have NaN density values for one or more cell types because they are not
     covered by the selected slices of the volumetric gene marker intensities.
     """
+    Path(fitted_densities_output_path).parent.mkdir(parents=True, exist_ok=True)
 
     L.info("Loading annotation ...")
     annotation = VoxelData.load_nrrd(annotation_path)
@@ -799,26 +904,30 @@ def fit_average_densities(
     voxel_data += list(gene_voxeldata.values())
     assert_properties(voxel_data)
 
-    with open(config["realignedSlicesPath"], "r", encoding="utf-8") as file_:
-        slices = json.load(file_)
-
-    with open(config["cellDensityStandardDeviationsPath"], "r", encoding="utf-8") as file_:
-        cell_density_stddev = json.load(file_)
-        cell_density_stddev = {
-            # Use the AIBS name attribute as key (this is a unique identifier in 1.json)
-            # (Ex: former key "|root|Basic cell groups and regions|Cerebrum" -> new key: "Cerebrum")
-            region_name.split("|")[-1]: stddev
-            for (region_name, stddev) in cell_density_stddev.items()
-        }
-
-    gene_marker_volumes = {
-        gene: {
-            "intensity": gene_data.raw,
-            "slices": slices[config["sectionDataSetID"][gene]],  # list of integer slice indices
-        }
-        for (gene, gene_data) in gene_voxeldata.items()
+    slices = utils.load_json(config["realignedSlicesPath"])
+    cell_density_stddev = utils.load_json(config["cellDensityStandardDeviationsPath"])
+    cell_density_stddev = {
+        # Use the AIBS name attribute as key (this is a unique identifier in 1.json)
+        # (Ex: former key "|root|Basic cell groups and regions|Cerebrum" -> new key: "Cerebrum")
+        name.split("|")[-1]: stddev
+        for (name, stddev) in cell_density_stddev.items()
     }
 
+    gene_marker_volumes = {}
+    for gene, gene_data in gene_voxeldata.items():
+        loc_slices = slices[config["sectionDataSetID"][gene]]
+        gene_marker_volumes[gene] = {
+            "intensity": gene_data.raw,
+            # list of integer slice indices
+            "slices": (
+                loc_slices - np.asarray(gene_data.offset / gene_data.voxel_dimensions, dtype=int)[0]
+                if loc_slices is not None
+                else None
+            ),
+        }
+
+    group_ids_config = utils.load_json(group_ids_config_path)
+
     L.info("Loading average densities dataframe ...")
     average_densities_df = pd.read_csv(average_densities_path)
     homogenous_regions_df = pd.read_csv(homogenous_regions_path)
@@ -832,9 +941,12 @@ def fit_average_densities(
         average_densities_df,
         homogenous_regions_df,
         cell_density_stddev,
+        region_name=region_name,
+        group_ids_config=group_ids_config,
+        min_data_points=min_data_points,
     )
 
-    # Turn index into column so as to ease off the save and load operations on csv files
+    # Turn index into column to ease off the save and load operations on csv files
     fitted_densities_df["brain_region"] = fitted_densities_df.index
 
     L.info("Saving fitted densities to file %s ...", fitted_densities_output_path)
@@ -847,6 +959,12 @@ def fit_average_densities(
 
 @app.command()
 @common_atlas_options
+@click.option(
+    "--region-name",
+    type=str,
+    default="root",
+    help="Name of the root region in the hierarchy",
+)
 @click.option(
     "--neuron-density-path",
     type=EXISTING_FILE_PATH,
@@ -880,6 +998,7 @@ def fit_average_densities(
 def inhibitory_neuron_densities(
     hierarchy_path,
     annotation_path,
+    region_name,
     neuron_density_path,
     average_densities_path,
     algorithm,
@@ -894,7 +1013,7 @@ def inhibitory_neuron_densities(
     (inhibitory neurons) and those of the inhibitory neuron subtypes PV+, SST+ and VIP+.
 
     The function modifies the estimates in `average_densities` (the "first estimates" of the paper)
-    to make them consistent accross cell types: the average density of GAD67+ cells in a leaf
+    to make them consistent across cell types: the average density of GAD67+ cells in a leaf
     region L should be at most the sum of the average densities of its subtypes under scrutiny
     (e.g. PV+, SST+ and VIP+) and should not exceed the neuron density of L.
 
@@ -919,14 +1038,15 @@ def inhibitory_neuron_densities(
     neuron_density = VoxelData.load_nrrd(neuron_density_path)
     if np.any(neuron_density.raw < 0.0):
         raise AtlasDensitiesError(f"Negative density value found in {neuron_density_path}.")
+
     L.info("Loading hierarchy ...")
-    with open(hierarchy_path, "r", encoding="utf-8") as file_:
-        hierarchy = json.load(file_)
-        if "msg" in hierarchy:
-            L.warning("Top-most object contains 'msg'; assuming AIBS JSON layout")
-            if len(hierarchy["msg"]) > 1:
-                raise AtlasDensitiesError("Unexpected JSON layout (more than one 'msg' child)")
-            hierarchy = hierarchy["msg"][0]
+
+    hierarchy = utils.load_json(hierarchy_path)
+    if "msg" in hierarchy:
+        L.warning("Top-most object contains 'msg'; assuming AIBS JSON layout")
+        if len(hierarchy["msg"]) > 1:
+            raise AtlasDensitiesError("Unexpected JSON layout (more than one 'msg' child)")
+        hierarchy = hierarchy["msg"][0]
 
     # Consistency check
     L.info("Checking consistency ...")
@@ -945,6 +1065,7 @@ def inhibitory_neuron_densities(
         _get_voxel_volume_in_mm3(annotation),
         neuron_density.raw,
         average_densities_df,
+        region_name=region_name,
     )
 
     L.info("Create inhibitory neuron densities: finished")
