@@ -25,7 +25,6 @@ Note that excitatory mtypes are handled in the first but not in the second case.
 """
 import json
 import logging
-import re
 from pathlib import Path
 from typing import Dict, List
 
@@ -40,18 +39,21 @@ from atlas_commons.app_utils import (
     common_atlas_options,
     log_args,
     set_verbose,
+    verbose_option,
 )
-from atlas_commons.utils import assert_metadata_content
 from voxcell import RegionMap, VoxelData  # type: ignore
 
 from atlas_densities.app.utils import AD_PATH, DATA_PATH
 from atlas_densities.densities.mtype_densities_from_composition import (
     create_from_composition as _create_from_composition,
 )
-from atlas_densities.densities.mtype_densities_from_map import check_probability_map_sanity
-from atlas_densities.densities.mtype_densities_from_map import (
+from atlas_densities.densities.mtype_densities_from_map.create import (
     create_from_probability_map as create_from_map,
 )
+from atlas_densities.densities.mtype_densities_from_map.utils import (
+    SYNAPSE_CLASSES,
+    check_probability_map_sanity,
+)
 from atlas_densities.densities.mtype_densities_from_profiles import DensityProfileCollection
 from atlas_densities.exceptions import AtlasDensitiesError
 
@@ -65,7 +67,7 @@ L = logging.getLogger(__name__)
 
 
 @click.group()
-@click.option("-v", "--verbose", count=True)
+@verbose_option
 def app(verbose):
     """Run the mtype densities CLI"""
     set_verbose(L, verbose)
@@ -199,92 +201,49 @@ def _check_config_sanity(config: dict) -> None:
         )
 
 
-def standardize_probability_map(probability_map: "pd.DataFrame") -> "pd.DataFrame":
-    """
-    Standardize the labels of the rows and the columns of `probability_map` and
-    remove unused rows.
-
-    Output labels are all lower case.
-    The underscore is the only delimiter used in an output label.
-    The layer names refered to by output labels are:
-        "layer_1," "layer_23", "layer_4", "layer_5" and "layer_6".
-    Rows whose labels contain "VIP" or "6b" are removed.
-
-    Row example: "L2/3 Pvalb-IRES-Cre" -> "layer_23_pv"
-    Column example: "NGC-SA" -> "ngc_sa"
-
-    Args:
-        probability_map: probability_map:
-            data frame whose rows are labeled by molecular types and layers (e.g.,
-            "L6a Htr3a-Cre_NO152", "L2/3 Pvalb-IRES-Cre", "L4 Htr3a-Cre_NO152") and whose columns
-            are labeled by mtypes (mtypes = morphological types, e.g., "NGC-SA", "ChC", "DLAC").
-
-    Returns: a data frame complying with all the above constraints.
-    """
-
-    def standardize_row_label(row_label: str):
-        """
-        Lowercase labels and use explicit layer names.
-        """
-        splitting = re.split("_", row_label)  # remove unused Creline information
-        splitting[0] = splitting[0].replace("L", "layer_")
-        splitting[1] = splitting[1].replace("Pvalb", "pv")
-        # Although Gad2 = Gad65, see e.g. https://www.genecards.org/cgi-bin/carddisp.pl?gene=GAD2
-        # Gad1 = Gad67 is taken as an acceptable substitute for density estimates.
-        splitting[1] = splitting[1].replace("Gad2", "gad67")
-
-        return "_".join(splitting).lower()
-
-    def standardize_column_label(col_label: str):
-        """
-        Lowercase labels and use underscore as delimiter for composed
-        molecular types such as NGC-DA or NGC-SA.
-
-        Example: "NGC-SA" -> "ngc_sa"
-        """
-        col_label = col_label.replace("-", "_")
-
-        return col_label.lower()
-
-    bbp_mtypes_map = {"DLAC": "LAC", "SLAC": "SAC"}
-    probability_map.rename(bbp_mtypes_map, axis="columns", inplace=True)
-    probability_map.rename(standardize_column_label, axis="columns", inplace=True)
-    probability_map.rename(standardize_row_label, axis="rows", inplace=True)
-
-    return probability_map
-
-
 @app.command()
 @common_atlas_options
 @click.option(
-    "--metadata-path",
+    "--probability-map",
     type=EXISTING_FILE_PATH,
-    required=False,
-    help=(
-        "(Optional) Path to the metadata json file. Defaults to "
-        f"`{str(METADATA_REL_PATH / 'isocortex_metadata.json')}`"
-    ),
-    default=str(METADATA_PATH / "isocortex_metadata.json"),
+    multiple=True,
+    required=True,
+    help=("Path to the probability map csv file."),
 )
 @click.option(
-    "--mtypes-config-path",
-    type=EXISTING_FILE_PATH,
+    "--marker",
+    type=(str, EXISTING_FILE_PATH),
+    multiple=True,
     required=True,
-    help="Path to the yaml configuration file. "
-    f"See `{str(MTYPES_PROBABILITY_MAP_REL_PATH / 'README.rst')}` for an example.",
+    help="Name and path to marker: ex: --marker pv path/pv.nrrd",
+)
+@click.option(
+    "--synapse-class",
+    type=click.Choice(list(SYNAPSE_CLASSES), case_sensitive=False),
+    required=True,
+    help="Target synapse class, the other will be skipped.",
 )
 @click.option(
     "--output-dir",
     required=True,
     help="Path to output directory. It will be created if it doesn't exist already.",
 )
+@click.option(
+    "--n-jobs",
+    required=False,
+    default=10,
+    type=int,
+    help="Number of jobs to run in parallel.",
+)
 @log_args(L)
-def create_from_probability_map(
+def create_from_probability_map(  # pylint: disable=too-many-arguments
     annotation_path,
     hierarchy_path,
-    metadata_path,
-    mtypes_config_path,
+    probability_map,
+    marker,
+    synapse_class,
     output_dir,
+    n_jobs,
 ):  # pylint: disable=too-many-locals
     """
     Create neuron density nrrd files for the mtypes listed in the probability mapping csv file.
@@ -299,27 +258,15 @@ def create_from_probability_map(
 
     Note: this command does not generate volumetric density files for excitatory neurons.
     """
-
-    L.info("Loading configuration file ...")
-    with open(mtypes_config_path, "r", encoding="utf-8") as file_:
-        config = yaml.load(file_, Loader=yaml.FullLoader)
-    _check_config_sanity(config)
-
-    L.info("Loading probability mapping ...")
-    probability_map = pd.DataFrame(pd.read_csv(config["probabilityMapPath"]))
-    if "molecular_type" in probability_map.columns:
-        probability_map.set_index("molecular_type", inplace=True)
-    else:
-        probability_map.set_index(probability_map.columns[0], inplace=True)
-
-    # Remove useless lines, use lower case and "standardized" explicit label names
-    probability_map = standardize_probability_map(probability_map)
-    check_probability_map_sanity(probability_map)
-
-    L.info("Loading brain region metadata ...")
-    with open(metadata_path, "r", encoding="utf-8") as file_:
-        metadata = json.load(file_)
-        assert_metadata_content(metadata)
+    probability_maps = []
+    for probability_map_path in probability_map:
+        L.info("Loading probability map %s", probability_map_path)
+        loaded_probability_map = pd.read_csv(probability_map_path)
+        loaded_probability_map.set_index(
+            ["region", "molecular_type", "synapse_class"], inplace=True
+        )
+        check_probability_map_sanity(loaded_probability_map)
+        probability_maps.append(loaded_probability_map)
 
     L.info("Loading hierarchy json file ...")
     region_map = RegionMap.load_json(hierarchy_path)
@@ -329,8 +276,7 @@ def create_from_probability_map(
 
     L.info("Loading volumetric densities of molecular types ...")
     molecular_type_densities = {
-        molecular_type: VoxelData.load_nrrd(density_path)
-        for (molecular_type, density_path) in config["molecularTypeDensityPaths"].items()
+        molecular_type: VoxelData.load_nrrd(density_path) for molecular_type, density_path in marker
     }
 
     # Check metadata consistency
@@ -341,13 +287,14 @@ def create_from_probability_map(
     create_from_map(
         annotation,
         region_map,
-        metadata,
         {
             molecular_type: density.raw
             for (molecular_type, density) in molecular_type_densities.items()
         },
-        probability_map,
+        probability_maps,
+        synapse_class,
         output_dir,
+        n_jobs,
     )
 
 
@@ -461,7 +408,7 @@ def create_from_composition(
 
 def _load_neuronal_mtype_taxonomy(filename: str) -> pd.DataFrame:
     """Loads the taxonomy tsv file into a dataframe"""
-    return pd.read_csv(filename, header=0, delim_whitespace=True)
+    return pd.read_csv(filename, header=0, sep=r"\s+")
 
 
 def _validate_mtype_taxonomy(taxonomy: pd.DataFrame) -> None:
@@ -502,7 +449,6 @@ def _load_neuronal_mtype_composition(filename: str) -> pd.DataFrame:
     composition_dict: Dict[str, List] = {"density": [], "layer": [], "mtype": []}
 
     for entry in composition:
-
         traits = entry["traits"]
         composition_dict["density"].append(entry["density"])
         composition_dict["layer"].append(f"layer_{traits['layer']}")
