@@ -16,8 +16,8 @@ between output cell counts and input estimates while enforcing the consistency o
 across the brain region hierarchy.
 
 The file ``doc/source/bbpp82_628_linear_program.pdf`` serves as a documentation of the linear
-program, see Section 2 in particular. The initalization of the linear program relies on average
-density estimates, obtained obtained for instance via the ``fitting`` module. We refer to these
+program, see Section 2 in particular. The initialization of the linear program relies on average
+density estimates, obtained for instance via the ``fitting`` module. We refer to these
 estimates as the initial estimates.
 
 The output are volumetric density nrrd files, one for each of the aforementioned inhibitory
@@ -36,29 +36,22 @@ and with NaN otherwise. If ``x_result.at[(id_, marker)]`` is set to NaN, then a
 variable deltas used in this module corresponds to the decision variables ``delta_{r, m}`` of
 the pdf file.
 """
+from __future__ import annotations
 
 import logging
 import warnings
-from typing import Dict, List, Tuple
+from typing import Dict, List, Optional, Set, Tuple
 
 import numpy as np
 import pandas as pd
 from atlas_commons.typing import AnnotationT, FloatArray
 from scipy.optimize import linprog
-from tqdm import tqdm
-from voxcell import RegionMap
+from voxcell import RegionMap, voxel_data
 
+from atlas_densities.densities import utils
 from atlas_densities.densities.inhibitory_neuron_densities_helper import (
     average_densities_to_cell_counts,
-    check_region_counts_consistency,
     get_cell_types,
-    replace_inf_with_none,
-    resize_average_densities,
-)
-from atlas_densities.densities.utils import (
-    compute_region_cell_counts,
-    compute_region_volumes,
-    get_hierarchy_info,
 )
 from atlas_densities.exceptions import AtlasDensitiesError, AtlasDensitiesWarning
 
@@ -66,6 +59,158 @@ L = logging.getLogger(__name__)
 
 SKIP = np.inf  # Used to signify that a delta variable of the linear program should be removed
 KEEP = np.nan  # Used to signify that a delta variable should be added to the linear program
+MinMaxPair = Tuple[float, Optional[float]]
+
+
+def _resize_average_densities(
+    average_densities: pd.DataFrame, hierarchy_info: pd.DataFrame
+) -> pd.DataFrame:
+    """
+    Resize the `average_densities` data frame in such a way that the resized index coincides with
+    `hierarchy_info["brain_region"]`.
+
+    Missing entries are set to ``np.nan``.
+
+    average_densities: a data frame whose columns are described in
+            :func:`atlas_densities.densities.fitting.linear_fitting` containing the average
+            cell densities of brain regions and their associated standard deviations. Columns are
+            labelled by T and T_standard_deviation for various cell types T. The index of
+            `average_densities` is a list of region names.
+    hierarchy_info: data frame returned by
+        :func:`atlas_densities.densities.utils.get_hierarchy_info`.
+
+    Returns: a data frame containing all the entries of `average_densities` and whose index is
+        `hierarchy_info["brain_region"]`. New entries are set to ``np.nan``.
+
+    """
+    resized = pd.DataFrame(
+        np.nan, index=hierarchy_info["brain_region"], columns=average_densities.columns
+    )
+    resized.loc[average_densities.index] = average_densities
+
+    return resized
+
+
+def _check_region_counts_consistency(
+    region_counts: pd.DataFrame, hierarchy_info: pd.DataFrame, tolerance: float = 0.0
+) -> None:
+    """
+    Check that cell counts considered as certain are consistent across the brain regions hierarchy.
+
+    Args:
+        region_counts: data frame returned by
+            :fun:`densities.inhibitory_densities_helper.average_densities_to_cell_counts`.
+            A region is understood as a region of the brain hierarchy and includes all descendant
+            subregions.
+        hierarchy_info: data frame returned by
+            :func:`atlas_densities.densities.utils.get_hierarchy_info`.
+        tolerance: non-negative float number used as tolerance when comparing counts.
+            Defaults to 0.0.
+
+    Raises:
+        AtlasDensitiesError if the sum of the cell counts of some leaf regions, all given with
+        certainty, exceeds the cell count of an ancestor, also given with certainty.
+
+    """
+
+    def _check_descendants_consistency(region_name: str, id_set: Set[int], cell_type: str):
+        standard_deviation = f"{cell_type}_standard_deviation"
+
+        if region_counts.at[region_name, standard_deviation] != 0.0:
+            return
+
+        # count is certain, since standard_deviation is 0
+        count = region_counts.at[region_name, cell_type]
+        descendants_names = hierarchy_info.loc[list(id_set), "brain_region"]
+        zero_std_mask = region_counts.loc[descendants_names, standard_deviation] == 0.0
+        mask = zero_std_mask & (region_counts.loc[descendants_names, cell_type] > count + tolerance)
+        descendants_counts = region_counts.loc[descendants_names, cell_type][mask]
+        if not descendants_counts.empty:
+            raise AtlasDensitiesError(
+                f"Counts of {cell_type} cells in regions {list(descendants_names)} all exceed "
+                f"the count of its ancestor region {region_name} and each count is given "
+                f"for certain. The counts {descendants_counts} are all larger than "
+                f"{count}."
+            )
+
+        names_with_certainty = descendants_names[zero_std_mask.to_list()].to_list()
+        leaf_names = [
+            hierarchy_info.at[id_, "brain_region"]
+            for id_ in id_set
+            if len(hierarchy_info.at[id_, "descendant_ids"]) == 1
+            and hierarchy_info.at[id_, "brain_region"] in names_with_certainty
+        ]
+        leaves_count_sum = np.sum(region_counts.loc[leaf_names, cell_type])
+        if leaves_count_sum > count + tolerance:
+            raise AtlasDensitiesError(
+                f"The sum of the counts of {cell_type} cells in leaf regions",
+                " which are given with certainty, exceeds the count of the ancestor"
+                f" region {region_name}, also given with certainty: "
+                f"{leaves_count_sum} > {count}.",
+            )
+
+    for _, region_name, id_set in hierarchy_info[["brain_region", "descendant_ids"]].itertuples():
+        for cell_type in get_cell_types(region_counts):
+            _check_descendants_consistency(region_name, id_set, cell_type)
+
+
+def _replace_inf_with_none(bounds: FloatArray) -> List[MinMaxPair]:
+    """
+    Replace the upper bounds equal to ``np.inf`` by None so as to comply with
+    the `bounds` interface of scipy.optimize.linprog.
+
+    Args:
+        bounds: float array of shape (N, 2). Values in `bounds[..., 1]` can be
+            ``np.inf`` to indicate the absence of a constraining upper bound.
+
+    Return:
+        list of pairs (min, max) where min is a float and max either a float or None.
+    """
+    return [(float(min_), None if np.isinf(max_) else float(max_)) for min_, max_ in bounds]
+
+
+def _compute_region_cell_counts(
+    annotation: AnnotationT,
+    density: FloatArray,
+    voxel_volume: float,
+    hierarchy_info: "pd.DataFrame",
+) -> "pd.DataFrame":
+    """
+    Compute the cell count of every 3D region in `annotation` labeled by a unique
+    id in `ids` wrt cell `density`.
+
+    Args:
+        annotation: int array of shape (W, H, D) holding the annotation of the whole AIBS
+            mouse brain. (The integers W, H and D are the dimensions of the array).
+        density: float array of shape `annotation.shape` holding the volumetric density of a
+            given cell type. The value assigned to a voxel represents the cell density in this
+            voxel expressed in number of cells per mm^3.
+        voxel_volume: volume in mm^3 of a voxel in any of the volumetric input arrays.
+            This is (25 * 1e-6) ** 3 for an AIBS atlas nrrd file with 25um resolution.
+        hierarchy_info: data frame returned by
+            :func:`atlas_densities.densities.utils.get_hierarchy_info`.
+
+    Returns:
+        DataFrame of the following form (values are fake):
+             brain_region                    cell_count
+        10   Basic cell groups and regions   200.30
+        123  Cerebrum                         75.05
+             ...                             ...
+        The index is the sorted list of all region identifiers.
+    """
+    vtiv = voxel_data.ValueToIndexVoxels(annotation)
+    density = vtiv.ravel(density)
+
+    id_counts = []
+    for id_ in hierarchy_info.index:
+        indices = vtiv.value_to_1d_indices(id_)
+        id_counts.append(np.sum(density[indices]) * voxel_volume)
+
+    result = pd.DataFrame(
+        {"brain_region": hierarchy_info["brain_region"], "cell_count": id_counts},
+        index=hierarchy_info.index,
+    )
+    return result
 
 
 def set_known_values(
@@ -124,7 +269,7 @@ def set_known_values(
         Zeroes the count of `cell_type` cells in every descendant regions of `id_`,
         including `id_`.
         """
-        desc_ids = list(hierarchy_info.at[id_, "descendant_id_set"])
+        desc_ids = list(hierarchy_info.at[id_, "descendant_ids"])
         x_result.loc[desc_ids, cell_type] = 0.0
         desc_names = hierarchy_info.loc[desc_ids, "brain_region"]
         deltas.loc[desc_names, cell_type] = 0.0
@@ -142,8 +287,8 @@ def set_known_values(
     L.info("Setting known values ...")
     # Set delta_{r, m} with `SKIP` if the corresponding cell count estimate is missing.
     # This means that the region r won't impose any constraint wrt to the marker m in the linear
-    # program, i. e., the delta_{r, m} variable is omitted for such r and m.
-    for (id_, region_name) in zip(hierarchy_info.index, region_counts.index):
+    # program, i.e., the delta_{r, m} variable is omitted for such r and m.
+    for id_, region_name in zip(hierarchy_info.index, region_counts.index):
         for cell_type in cell_types:
             # A region without cell count estimate for `cell_type` adds no constraint
             # to the linear program.
@@ -160,7 +305,7 @@ def set_known_values(
                 _zero_descendants(id_, cell_type, x_result, deltas, hierarchy_info)
 
     # Set the (possibly non-zero) cell count estimates which are given with certainty.
-    for (id_, region_name) in zip(hierarchy_info.index, region_counts.index):
+    for id_, region_name in zip(hierarchy_info.index, region_counts.index):
         for cell_type in cell_types:
             if np.isclose(
                 region_counts.at[region_name, cell_type + "_standard_deviation"],
@@ -209,7 +354,7 @@ def create_bounds(
             bound.
         - `x_map` is a dict mapping pairs (id_, cell_type) to indices of `bounds`.
             The keys of this dict corresponds to cell count variables in the linear program.
-        - `deltas_map` is a dict mapping pairs (region_name, cell_type) to to indices of `bounds`.
+        - `deltas_map` is a dict mapping pairs (region_name, cell_type) to indices of `bounds`.
             The keys of this dict corresponds to the slack "delta" variables which represent
             distances of cell count variables to initial estimates.
     """
@@ -290,7 +435,7 @@ def create_aub_and_bub(
         constraint: FloatArray, b_value: float, region_name: str, id_: int
     ) -> bool:
         """
-        Check if `contraint` is a valid inequality constraint of type (3e).
+        Check if `constraint` is a valid inequality constraint of type (3e).
 
         Returns:
             True if the constraint is not void.
@@ -319,9 +464,7 @@ def create_aub_and_bub(
     b_ub = []
     variable_count = len(x_map) + len(deltas_map)
     cell_types = get_cell_types(region_counts)
-    for (id_, region_name, set_) in zip(
-        hierarchy_info.index, hierarchy_info["brain_region"], hierarchy_info["descendant_id_set"]
-    ):
+    for id_, region_name, set_ in hierarchy_info[["brain_region", "descendant_ids"]].itertuples():
         for cell_type in cell_types:
             if (region_name, cell_type) in deltas_map:
                 constraint = np.zeros((variable_count,), dtype=float)
@@ -387,18 +530,21 @@ def create_volumetric_densities(
         `annotation.shape` (which is assumed to coincide with `neuron_density.shape`). The value
         of a voxel in each array represents a cell density expressed in number of cells per mm^3.
     """
+    index = voxel_data.ValueToIndexVoxels(annotation)
 
-    densities = {cell_type: neuron_density.copy() for cell_type in cell_types}
-    for id_ in tqdm(neuron_counts.index):
-        mask = annotation == id_
+    raveled_densities = {cell_type: index.ravel(neuron_density.copy()) for cell_type in cell_types}
+    for id_ in neuron_counts.index:
+
+        idx = index.value_to_1d_indices(id_)
         neuron_count = neuron_counts.at[id_, "cell_count"]
+
         for cell_type in cell_types:
             if np.isclose(neuron_count, 0.0):
-                densities[cell_type][mask] = 0.0
+                raveled_densities[cell_type][idx] = 0.0
             else:
-                densities[cell_type][mask] *= x_result.at[id_, cell_type] / neuron_count
+                raveled_densities[cell_type][idx] *= x_result.at[id_, cell_type] / neuron_count
 
-    return densities
+    return {key: index.unravel(value) for key, value in raveled_densities.items()}
 
 
 def _compute_initial_cell_counts(
@@ -429,20 +575,18 @@ def _compute_initial_cell_counts(
             :fun:`densities.inhibitory_densities_helper.average_densities_to_cell_counts`.
             It holds region cell counts as well as associated standard deviations.
         - `id_counts` has the return value format of
-            :fun:`densities.inhbitory_densities_helper.average_densities_to_cell_counts` except
+            :fun:`densities.inhibitory_densities_helper.average_densities_to_cell_counts` except
             that its index is a list of unique integer region identifiers.
             It holds the cell counts of the 3D regions labeled by a unique identifier, as well as
             associated standard deviations (descendant subregions are excluded).
     """
     L.info("Computing the volume of every 3D region ...")
-    volumes = compute_region_volumes(annotation, voxel_volume, hierarchy_info)
+    volumes = utils.compute_region_volumes(annotation, voxel_volume, hierarchy_info)
 
     L.info("Computing cell count estimates in every 3D region ...")
     region_counts = average_densities_to_cell_counts(average_densities, volumes)
 
-    # Detect cell counts inconsistency between a region and its descendants when
-    # estimates are deemed as certain.
-    check_region_counts_consistency(region_counts, hierarchy_info)
+    _check_region_counts_consistency(region_counts, hierarchy_info)
 
     L.info("Computing cell count estimates in atomic 3D region ...")
     volumes.drop("volume", axis=1, inplace=True)
@@ -455,22 +599,16 @@ def _compute_initial_cell_counts(
 
 def _check_variables_consistency(
     x_result: pd.DataFrame,
-    deltas: pd.DataFrame,
     cell_types: List[str],
     neuron_counts: pd.DataFrame,
     hierarchy_info: pd.DataFrame,
 ) -> None:
     """
-    Raises an error if a delta variable indexed by (region_name, cell_type) is set with a non-NaN
-    value whereas an x_result variable indexed by (desc_id, cell_type) is set with a NaN value for
-    some descendant id of region_name.
-
     Raises also an error if a cell count value marked as final in `x_result` exceeds its
     prescribed upper bound from `neuron_counts`.
 
     Args:
         x_result: see return value of :fun:`set_known_values`.
-        deltas: idem
         cell_types: list of cell type names, e.g., ["pv+", "sst+", "vip+", "gad67+"].
         hierarchy_info: data frame returned by
             :func:`atlas_densities.densities.utils.get_hierarchy_info`.
@@ -478,24 +616,12 @@ def _check_variables_consistency(
             region labeled by an integer identifier in `neuron_count.index`.
 
     Raises:
-        AtlasDensitiesError if on the the following assumptions is violated:
-        - if cell count estimate of a region is known with certainty for a given cell type,
-        then the cell count of every descendant region is also known with certainty.
-        - a cell count estimate which is given for certain does not
+        AtlasDensitiesError if a cell count estimate which is given for certain exceeds the neuron
+        count constraint.
     """
     cell_count_tolerance = 1e-2  # absolute tolerance to rule out round-off errors
-    for region_name, id_, id_set in zip(
-        deltas.index, hierarchy_info.index, hierarchy_info["descendant_id_set"]
-    ):
+    for id_ in hierarchy_info.index:
         for cell_type in cell_types:
-            if np.isfinite(deltas.loc[region_name, cell_type]):
-                for desc_id in id_set:
-                    if np.isnan(x_result.loc[desc_id, cell_type]):
-                        raise AtlasDensitiesError(
-                            f"Cell count estimate of region named '{region_name}' for cell type "
-                            f"{cell_type} was given for certain whereas the cell count of "
-                            f"descendant id {desc_id} is not certain."
-                        )
             neuron_count = neuron_counts.loc[id_, "cell_count"]
             if (
                 not np.isnan(x_result.loc[id_, cell_type])
@@ -552,6 +678,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
     voxel_volume: float,
     neuron_density: FloatArray,
     average_densities: pd.DataFrame,
+    region_name: str = "root",
 ) -> Dict[str, FloatArray]:
     """
     Create a 3D float array for each cell type that labels a column of `average_densities`.
@@ -598,6 +725,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
             cell densities of brain regions and their associated standard deviations. Columns are
             labelled by T and T_standard_deviation for various cell types T. The index of
             `average_densities` is a list of region names.
+        region_name: (str) name of the root region of interest
 
     Returns:
         a dict whose keys are cell types (e.g., "gad67+", "pv+", "sst+" and "vip+") and whose
@@ -608,27 +736,26 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
         AtlasDensitiesError if some inconsistency in the input data has been detected or if the
         linear program cannot be solved.
     """
+    hierarchy_info = utils.get_hierarchy_info(RegionMap.from_dict(hierarchy), root=region_name)
+    average_densities = _resize_average_densities(average_densities, hierarchy_info)
 
-    hierarchy_info = get_hierarchy_info(RegionMap.from_dict(hierarchy), root="root")
-    average_densities = resize_average_densities(average_densities, hierarchy_info)
+    L.info("Retrieving overall neuron counts in atomic 3D regions ...")
+    neuron_counts = _compute_region_cell_counts(
+        annotation, neuron_density, voxel_volume, hierarchy_info
+    )
+    assert np.all(neuron_counts["cell_count"] >= 0.0)
 
     L.info("Initialization of the linear program: started")
     region_counts, id_counts = _compute_initial_cell_counts(
         annotation, voxel_volume, average_densities, hierarchy_info
     )
 
-    L.info("Retrieving overall neuron counts in atomic 3D regions ...")
-    neuron_counts = compute_region_cell_counts(
-        annotation, neuron_density, voxel_volume, hierarchy_info, with_descendants=False
-    )
-    assert np.all(neuron_counts["cell_count"] >= 0.0)
-
     L.info("Setting the known values ...")
     x_result, deltas = set_known_values(region_counts, id_counts, hierarchy_info)
     # We assume that if the cell count of `cell_type` in `region_name` is known with certainty
     # then the same holds in every descendant subregion.
     _check_variables_consistency(
-        x_result, deltas, get_cell_types(region_counts), neuron_counts, hierarchy_info
+        x_result, get_cell_types(region_counts), neuron_counts, hierarchy_info
     )
 
     L.info("Setting variable bounds and further inequality constraints ...")
@@ -646,9 +773,9 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
     L.info("Initialization of the linear program: finished.")
     if variable_count != 0:  # linprog raises a ValueError if c_row is empty
         c_row = np.zeros((variable_count,), dtype=float)
-        for (region_name, cell_type), index in deltas_map.items():
+        for (name, cell_type), index in deltas_map.items():
             std_name = cell_type + "_standard_deviation"
-            c_row[index] = 1.0 / region_counts.at[region_name, std_name]
+            c_row[index] = 1.0 / region_counts.at[name, std_name]
             assert c_row[index] > 0.0
 
         L.info(
@@ -660,7 +787,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
             c=c_row,
             A_ub=a_ub,
             b_ub=b_ub,
-            bounds=replace_inf_with_none(bounds),
+            bounds=_replace_inf_with_none(bounds),
             method="highs",
         )
         if not result.success:
