@@ -12,6 +12,7 @@ This code supersedes the inhibitory_neuron_densities module, which is maintained
 comparisons with the former approach of "A Cell Atlas for the Mouse Brain" by C. Eroe et
 al., 2018.
 """
+from __future__ import annotations
 
 import copy
 import logging
@@ -24,6 +25,7 @@ from voxcell import RegionMap
 
 from atlas_densities.densities.inhibitory_neuron_densities_helper import (
     average_densities_to_cell_counts,
+    get_cell_types,
 )
 from atlas_densities.densities.utils import compute_region_volumes, get_hierarchy_info
 
@@ -368,7 +370,8 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
     annotation: AnnotationT,
     voxel_volume: float,
     neuron_density: FloatArray,
-    average_densities: "pd.DataFrame",
+    average_densities: pd.DataFrame,
+    region_name: str = "root",
 ) -> Dict[str, FloatArray]:
     """
     Create a 3D float array for each cell type labelling the columns of `average_densities`.
@@ -409,6 +412,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
             cell densities of brain regions and their associated standard deviations. Columns are
             labelled by T and T_standard_deviation for various cell types T. The index of
             `average_densities` is a list of region names.
+        region_name: (str) name of the root region of interest
 
     Returns:
         a dict whose keys are cell types (e.g., "gad67+", "pv+", "sst+" and "vip+") and whose
@@ -418,7 +422,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
     """
 
     region_map = RegionMap.from_dict(hierarchy)
-    hierarchy_info = get_hierarchy_info(region_map, root="root")
+    hierarchy_info = get_hierarchy_info(region_map, root=region_name)
     L.info("Computing region volumes ...")
     volumes = compute_region_volumes(annotation, voxel_volume, hierarchy_info)
 
@@ -426,7 +430,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
     # modify cell densities recursively starting from the leaves
     L.info("Computing region cell counts ...")
     region_counts = average_densities_to_cell_counts(average_densities, volumes)
-    cell_types = {column.replace("_standard_deviation", "") for column in region_counts.columns}
+    cell_types = get_cell_types(region_counts)
 
     density_helper = VolumetricDensityHelper(
         annotation,
@@ -434,7 +438,7 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
         neuron_density,
         region_counts,
         "gad67+",
-        list(cell_types - {"gad67+"}),
+        list(set(cell_types) - {"gad67+"}),
     )
 
     L.info("Making leaf region cell counts consistent ...")
@@ -452,13 +456,13 @@ def create_inhibitory_neuron_densities(  # pylint: disable=too-many-locals
 
     L.info("Filling leaf region volumetric densities ...")
     for region_id in tqdm(hierarchy_leaf_info.index):
-        region_name = hierarchy_leaf_info.brain_region[region_id]
-        if region_name in leaf_region_counts:  # Some leaf regions may be missing
-            density_helper.fill_volumetric_densities(region_id, leaf_region_counts[region_name])
+        name = hierarchy_leaf_info.brain_region[region_id]
+        if name in leaf_region_counts:  # Some leaf regions may be missing
+            density_helper.fill_volumetric_densities(region_id, leaf_region_counts[name])
 
     L.info("Filling remaining volumetric densities ...")
-    for region_name in tqdm(nomansland_counts):
-        region_id = region_map.find(region_name, attr="name").pop()
-        density_helper.fill_volumetric_densities(region_id, nomansland_counts[region_name])
+    for name in tqdm(nomansland_counts):
+        region_id = region_map.find(name, attr="name").pop()
+        density_helper.fill_volumetric_densities(region_id, nomansland_counts[name])
 
     return density_helper.volumetric_densities
