@@ -1,6 +1,10 @@
 """Utility functions for cell density computation."""
+from __future__ import annotations
 
-from typing import TYPE_CHECKING, Dict, Set
+import json
+from copy import deepcopy
+from pathlib import Path
+from typing import TYPE_CHECKING
 from warnings import warn
 
 import numpy as np
@@ -8,7 +12,6 @@ import pandas as pd
 import scipy.misc
 import scipy.ndimage
 from atlas_commons.typing import AnnotationT, BoolArray, FloatArray
-from tqdm import tqdm
 
 from atlas_densities.exceptions import AtlasDensitiesError, AtlasDensitiesWarning
 from atlas_densities.utils import copy_array
@@ -17,20 +20,14 @@ if TYPE_CHECKING:  # pragma: no cover
     from voxcell import RegionMap  # type: ignore
 
 
-MEASUREMENT_TYPES = {
-    # For a given brain region R and a given cell type T:
-    0: "cell density",  # number of cells of type T per mm^3 in R
-    1: "neuron proportion",  # number of cells of type T / number of neurons in R
-    2: "cell proportion",  # number of cells of type T / number of cells in R
-    3: "cell count per slice",  # number of cells of type T per slice of R, see MEASUREMENT_UNITS
-}
+GROUP_IDS_PATH = Path(__file__).parent.parent / "app/data/metadata/group_ids.json"
 
-MEASUREMENT_UNITS = {
-    "cell density": "number of cells per mm^3",
-    "neuron proportion": "None",
-    "cell proportion": "None",
-    "cell count per slice": "number of cells per 5 micrometer-thick slice",
-}
+
+def load_json(path):
+    """Load a json file and return its decoded contents"""
+    with open(path, "r", encoding="utf-8") as fd:
+        ret = json.load(fd)
+    return ret
 
 
 def normalize_intensity(
@@ -70,11 +67,11 @@ def normalize_intensity(
         3D array, the normalized marker intensity array.
     """
 
-    outside_mean = np.mean(
-        marker_intensity[np.logical_and((annotation == region_id), (marker_intensity > 0.0))]
-    )
     output_intensity = copy_array(marker_intensity, copy=copy)
-    output_intensity -= outside_mean * threshold_scale_factor
+    mask_low_filter = np.logical_and((annotation == region_id), (marker_intensity > 0.0))
+    if np.any(mask_low_filter):
+        outside_mean = np.mean(marker_intensity[mask_low_filter])
+        output_intensity -= outside_mean * threshold_scale_factor
     output_intensity[output_intensity < 0.0] = 0.0
     max_intensity = np.max(output_intensity)
     if max_intensity > 0:
@@ -147,7 +144,7 @@ def compensate_cell_overlap(
 # pylint: disable=fixme
 # TODO: Re-assess and underline each density validation criterion. Design an actual optimization
 # strategy if appropriate.
-def optimize_distance_to_line(  # pylint: disable=too-many-arguments
+def _optimize_distance_to_line(  # pylint: disable=too-many-arguments
     line_direction_vector: FloatArray,
     upper_bounds: FloatArray,
     sum_constraint: float,
@@ -158,7 +155,7 @@ def optimize_distance_to_line(  # pylint: disable=too-many-arguments
     """
     Find inside a box the closest point to a line with prescribed coordinate sum.
 
-    This function aims at solving the following (convex quadratic) optmization problem:
+    This function aims at solving the following (convex quadratic) optimization problem:
 
     Given a sum S >= 0.0, a line D in the non-negative orthant of the Euclidean N-dimensional
     space and a box B in this orthant (an N-dimensional vector with non-negative coordinates),
@@ -176,26 +173,30 @@ def optimize_distance_to_line(  # pylint: disable=too-many-arguments
     Args:
         line_direction_vector: N-dimensional float vector with non-negative coordinates.
         upper_bounds: N-dimensional float vector with non-negative coordinates. Defines the
-            the box constraining the optimization process.
-        sum_constraints: non-negative float number. The coordinate sum constraints imposed on
+            box constraining the optimization process.
+        sum_constraint: non-negative float number. The coordinate sum constraints imposed on
             the point P we are looking for.
         threshold: non-negative float value. If the coordinate sum of the current point
             is below `threshold`, the function returns the current point.
         max_iter: maximum number of iterations.
-        copy: If True, the function makes a copy of the input `line_direction_vector`. Otherwise
+        copy: If True, the function makes a copy of the input `line_direction_vector`. Otherwise,
             `line_direction_vector` is modified in-place and holds the optimal value.
 
     Returns: N-dimensional float vector with non-negative coordinates. The solution point of the
-        optimization problem, if it exists, up to inacurracy due to threshold size or early
-        termination of the algorithm. Otherwise a point on the boundary of the box B defined by
+        optimization problem, if it exists, up to inaccuracy due to threshold size or early
+        termination of the algorithm. Otherwise, a point on the boundary of the box B defined by
         `upper_bounds`.
     """
     diff = float("inf")
     iter_ = 0
     point = line_direction_vector.copy() if copy else line_direction_vector
-    while diff > threshold and iter_ < max_iter:
-        point *= sum_constraint / np.sum(point)
+    scalable_voxels = point != 0
+    while diff > threshold and iter_ < max_iter and scalable_voxels.any():
+        point[scalable_voxels] *= (sum_constraint - np.sum(point[~scalable_voxels])) / np.sum(
+            point[scalable_voxels]
+        )
         point = np.min([point, upper_bounds], axis=0)
+        scalable_voxels = np.logical_and(point != 0, point < upper_bounds)
         diff = np.abs(np.sum(point) - sum_constraint)
         iter_ += 1
 
@@ -299,7 +300,7 @@ def constrain_cell_counts_per_voxel(  # pylint: disable=too-many-arguments, too-
 
     # Find a cell count field respecting all the constraints and which is as close
     # as possible to the line defined by the input cell counts wrt to Euclidean norm.
-    cell_counts[complement] = optimize_distance_to_line(
+    cell_counts[complement] = _optimize_distance_to_line(
         line_direction_vector,
         upper_bound,
         target_sum - max_subsum,
@@ -319,7 +320,90 @@ def constrain_cell_counts_per_voxel(  # pylint: disable=too-many-arguments, too-
     return cell_counts
 
 
-def get_group_ids(region_map: "RegionMap", cleanup_rest: bool = False) -> Dict[str, Set[int]]:
+def get_fiber_tract_ids(region_map: "RegionMap", config: dict) -> set[int]:
+    """Ibid.
+
+    Args:
+        region_map: object to navigate the mouse brain regions hierarchy
+        config: mapping of regions to their constituent ids
+    """
+    fiber_tracts_ids = get_group_ids(region_map, config, _skip_check=True)["Fiber tracts group"]
+    assert fiber_tracts_ids, "Missing ids in Fiber tracts"
+    return fiber_tracts_ids
+
+
+def get_purkinje_layer_ids(region_map: "RegionMap", config: dict) -> set[int]:
+    """Ibid.
+
+    Args:
+        region_map: object to navigate the mouse brain regions hierarchy
+        config: mapping of regions to their constituent ids
+    """
+
+    purkinje_layer_ids = get_group_ids(region_map, config, _skip_check=True)["Purkinje layer"]
+    assert purkinje_layer_ids, "Missing ids in Purkinje layer"
+    return purkinje_layer_ids
+
+
+UNION = "UNION"
+INTERSECT = "INTERSECT"
+REMOVE = "REMOVE"
+
+
+def _interpret_region_groups(region_map, config):
+    # key:
+    #  UNION - creates union of set of ids
+    #  INTERSECT - creates intersection of ids
+    #  REMOVE - removes ids
+    OPS = (
+        UNION,
+        INTERSECT,
+        REMOVE,
+    )
+    groups = {}
+
+    def make_query(query):
+        if isinstance(query, dict) and any(op in query for op in OPS):
+            ids = materialize(query)
+        elif isinstance(query, dict):
+            query = dict(query)
+            with_descendants = query.pop("with_descendants")
+            assert len(query) == 1
+            attr, value = next(iter(query.items()))
+            ids = region_map.find(value, attr=attr, with_descendants=with_descendants)
+        elif isinstance(query, str) and query[0] == "!":
+            ids = set(groups[query[1:]])
+        else:
+            raise ValueError(f"unknown query: {query}")
+        return ids
+
+    def materialize(node):
+        assert len(node) == 1
+        op, queries = next(iter(node.items()))
+        assert op in OPS, f"{op} is not in {OPS}"
+        if op == UNION:
+            ids = set()
+            for query in queries:
+                ids |= make_query(query)
+        elif op == INTERSECT:
+            ids = make_query(queries[0])
+            for query in queries[1:]:
+                ids &= make_query(query)
+        elif op == REMOVE:
+            ids = make_query(queries[0]) - make_query(queries[1])
+
+        return ids
+
+    config = deepcopy(config)
+    for node in config:
+        name = node.pop("name")
+        assert name not in groups
+        groups[name] = materialize(node)
+
+    return groups
+
+
+def get_group_ids(region_map: "RegionMap", config: dict, _skip_check=False) -> dict[str, set[int]]:
     """
     Get AIBS structure ids for several region groups of interest.
 
@@ -328,84 +412,22 @@ def get_group_ids(region_map: "RegionMap", cleanup_rest: bool = False) -> Dict[s
 
     Args:
         region_map: object to navigate the mouse brain regions hierarchy
-            (instantied from AIBS 1.json).
-        cleanup_rest: (Optional) If True, the id of any ascendant region of the Cerebellum and
-            Isocortex groups are removed from the ids of the Rest group. This makes sure that
-            the set of ids of the Rest group is closed under taking descendants.
-
+            (instantiated from AIBS 1.json).
+        config: mapping of regions to their constituent ids
     Returns:
         A dictionary whose keys are region group names and whose values are
         sets of structure identifiers.
     """
-    cerebellum_group_ids = region_map.find(
-        "Cerebellum", attr="name", with_descendants=True
-    ) | region_map.find("arbor vitae", attr="name", with_descendants=True)
-    isocortex_group_ids = (
-        region_map.find("Isocortex", attr="name", with_descendants=True)
-        | region_map.find("Entorhinal area", attr="name", with_descendants=True)
-        | region_map.find("Piriform area", attr="name", with_descendants=True)
-    )
-    purkinje_layer_ids = region_map.find("@.*Purkinje layer", attr="name", with_descendants=True)
-    fiber_tracts_ids = (
-        region_map.find("fiber tracts", attr="name", with_descendants=True)
-        | region_map.find("grooves", attr="name", with_descendants=True)
-        | region_map.find("ventricular systems", attr="name", with_descendants=True)
-        | region_map.find("Basic cell groups and regions", attr="name")
-        | region_map.find("Cerebellum", attr="name")
-    )
-    cerebellar_cortex_ids = region_map.find("Cerebellar cortex", attr="name", with_descendants=True)
-    molecular_layer_ids = cerebellar_cortex_ids & region_map.find(
-        "@.*molecular layer", attr="name", with_descendants=True
-    )
-    rest_ids = region_map.find("root", attr="name", with_descendants=True)
-    rest_ids -= cerebellum_group_ids | isocortex_group_ids
-
-    if cleanup_rest:
-        # Make the Rest group stable under taking descendants
-        isocortex_id = region_map.find("Isocortex", attr="name").pop()
-        cerebellum_id = region_map.find("Cerebellum", attr="name").pop()
-        ascendant_ids = set(region_map.get(isocortex_id, attr="id", with_ascendants=True))
-        ascendant_ids |= set(region_map.get(cerebellum_id, attr="id", with_ascendants=True))
-        rest_ids -= ascendant_ids
-
-    return {
-        "Cerebellum group": cerebellum_group_ids,
-        "Isocortex group": isocortex_group_ids,
-        "Fiber tracts group": fiber_tracts_ids,
-        "Purkinje layer": purkinje_layer_ids,
-        "Molecular layer": molecular_layer_ids,
-        "Cerebellar cortex": cerebellar_cortex_ids,
-        "Rest": rest_ids,
-    }
-
-
-def get_group_names(region_map: "RegionMap", cleanup_rest: bool = False) -> Dict[str, Set[str]]:
-    """
-    Get AIBS names for regions in several region groups of interest.
-
-    Args:
-        region_map: object to navigate the mouse brain regions hierarchy
-            (instantied from AIBS 1.json).
-        cleanup_rest: (Optional) If True, the name of any ascendant region of the Cerebellum and
-            Isocortex groups are removed from the names of the Rest group. This makes sure that
-            the set of names of the Rest group is closed under taking descendants.
-
-    Returns:
-        A dictionary whose keys are region group names and whose values are
-        sets of brain region names.
-    """
-
-    group_ids = get_group_ids(region_map, cleanup_rest)
-
-    return {
-        group_name: {region_map.get(id_, attr="name") for id_ in group_ids[group_name]}
-        for group_name in ["Cerebellum group", "Isocortex group", "Rest"]
-    }
+    ret = _interpret_region_groups(region_map, config)
+    if not _skip_check:
+        for name, ids in ret.items():
+            assert ids, f"Missing ids in {name}"
+    return ret
 
 
 def get_region_masks(
-    group_ids: Dict[str, Set[int]], annotation: FloatArray
-) -> Dict[str, BoolArray]:
+    group_ids: dict[str, set[int]], annotation: FloatArray
+) -> dict[str, BoolArray]:
     """
     Get the boolean masks of several region groups of interest.
 
@@ -430,32 +452,12 @@ def get_region_masks(
     }
 
 
-def get_aibs_region_names(region_map: "RegionMap") -> Set[str]:
-    """
-    Retrieve the names of every region in `region_map`.
-
-    Args:
-        region_map: RegionMap object to navigate the brain regions hierarchy
-            instantiated with the 1.json hierarchy file from AIBS.
-
-    Returns:
-        Set of strings containing the names of all regions represented in
-        `region_map`.
-
-    """
-    aibs_region_ids = region_map.find(
-        "Basic cell groups and regions", attr="name", with_descendants=True
-    )
-
-    return {region_map.get(id_, "name") for id_ in aibs_region_ids}
-
-
 def get_hierarchy_info(
     region_map: "RegionMap",
     root: str = "Basic cell groups and regions",
 ) -> "pd.DataFrame":
     """
-    Returns the name and the descendant_id_set of each region that can be found by `region_map`.
+    Returns the name and the descendant_ids of each region that can be found by `region_map`.
 
     Note: We assume that the hierarchy file has unique brain region names.
 
@@ -468,24 +470,23 @@ def get_hierarchy_info(
     Returns:
         returns a dataframe with index a list of unique region ids and two columns
         (values are fake)
-                 descendant_id_set    brain_region
+                 descendant_ids    brain_region
             1    {1, 3}               "Cerebellum"
             2    {2, 4, 10}           "Isocortex"
                  ...             ...
         The index consists in the sorted list of the identifiers of every region recorded in
-        `region_map` under root. The column `descendant_id_set` holds for each region the set of
+        `region_map` under root. The column `descendant_ids` holds for each region the set of
         identifiers of the descendants including the region itself. `brain region` is the list of
         every region name.
 
     """
     region_ids = list(region_map.find(root, attr="name", with_descendants=True))
     region_ids.sort()
-    descendant_id_sets = [
-        region_map.find(id_, attr="id", with_descendants=True) for id_ in region_ids
-    ]
+    descendant_ids = [region_map.find(id_, attr="id", with_descendants=True) for id_ in region_ids]
     region_names = [region_map.get(id_, attr="name") for id_ in region_ids]
     data_frame = pd.DataFrame(
-        {"brain_region": region_names, "descendant_id_set": descendant_id_sets}, index=region_ids
+        {"brain_region": region_names, "descendant_ids": descendant_ids},
+        index=region_ids,
     )
 
     return data_frame
@@ -520,113 +521,21 @@ def compute_region_volumes(
         The latter column is created only if `hierarchy_info` has no `id_set` column,
         in which case its index is made of unique integer identifiers.
     """
-    id_volumes = []
-    for id_ in tqdm(hierarchy_info.index):
-        id_volumes.append(np.count_nonzero(annotation == id_) * voxel_volume)
-
     result = pd.DataFrame(
         {
             "brain_region": hierarchy_info["brain_region"],
-            "id_volume": id_volumes,
+            "id_volume": 0.0,
         },
         index=hierarchy_info.index,
     )
 
+    ids, counts = np.unique(annotation, return_counts=True)
+    result.update({"id_volume": pd.Series(counts * voxel_volume, index=ids)})
+
     volumes = []
-    for id_ in tqdm(hierarchy_info.index):
-        id_set = hierarchy_info.loc[id_, "descendant_id_set"]
+    for id_ in hierarchy_info.index:
+        id_set = hierarchy_info.loc[id_, "descendant_ids"]
         volume = result.loc[list(id_set), "id_volume"].sum()
         volumes.append(volume)
     result["volume"] = volumes
-
     return result
-
-
-def compute_region_cell_counts(
-    annotation: AnnotationT,
-    density: FloatArray,
-    voxel_volume: float,
-    hierarchy_info: "pd.DataFrame",
-    with_descendants: bool = True,
-) -> "pd.DataFrame":
-    """
-    Compute the cell count of every 3D region in `annotation` labeled by a unique
-    id in `ids` wrt cell `density`.
-
-    Args:
-        annotation: int array of shape (W, H, D) holding the annotation of the whole AIBS
-            mouse brain. (The integers W, H and D are the dimensions of the array).
-        density: float array of shape `annotation.shape` holding the volumetric density of a
-            given cell type. The value assigned to a voxel represents the cell density in this
-            voxel expressed in number of cells per mm^3.
-        voxel_volume: volume in mm^3 of a voxel in any of the volumetric input arrays.
-            This is (25 * 1e-6) ** 3 for an AIBS atlas nrrd file with 25um resolution.
-        hierarchy_info: data frame returned by
-            :func:`atlas_densities.densities.utils.get_hierarchy_info`.
-        with_descendants: if True, a computed cell count refers to the entire 3D region
-            designated by a region name, including all descendants subregions. Otherwise, the
-            computed cell count is the cell count of the 3D region labeled by the unique region
-            identifier. Defaults to True.
-
-    Returns:
-        DataFrame of the following form (values are fake):
-             brain_region                    cell_count
-        10   Basic cell groups and regions   200.30
-        123  Cerebrum                         75.05
-             ...                             ...
-        The index is the sorted list of all region identifiers.
-    """
-    id_counts = []
-    for id_ in tqdm(hierarchy_info.index):
-        mask = annotation == id_
-        id_counts.append(np.sum(density[mask]) * voxel_volume)
-
-    result = pd.DataFrame(
-        {"brain_region": hierarchy_info["brain_region"], "cell_count": id_counts},
-        index=hierarchy_info.index,
-    )
-
-    if with_descendants:
-        counts = []
-        for id_set in tqdm(hierarchy_info["descendant_id_set"]):
-            count = result.loc[list(id_set), "cell_count"].sum()
-            counts.append(count)
-        result["cell_count"] = counts
-
-    return result
-
-
-def zero_negative_values(array: FloatArray) -> None:
-    """
-    Zero negative values resulting from round-off errors.
-
-    Modifies `array` in place.
-
-    Args:
-        array: float numpy array. We expect most of the `array` values to be non-negative.
-            Negative values should be negligible when compared to positive values.
-
-    Raises:
-        AtlasDensitiesError if the absolute value of the sum of all negative values exceeds
-            1 percent of the sum of all positive values, or if the smallest negative value is
-            not negligible wrt to the mean of the non-negative values.
-    """
-    negative_mask = array < 0.0
-    if np.count_nonzero(negative_mask) == 0:
-        return
-
-    non_negative_mask = np.invert(negative_mask)
-
-    if np.abs(np.sum(array[negative_mask])) / np.sum(array[non_negative_mask]) > 0.01:
-        raise AtlasDensitiesError(
-            "The absolute value of the sum of all negative values exceeds"
-            " 1 percent of the sum of all positive values"
-        )
-    ratio = np.abs(np.min(array[negative_mask])) / np.mean(array[non_negative_mask])
-    if not np.isclose(ratio, 0.0, atol=1e-08):
-        raise AtlasDensitiesError(
-            "The smallest negative value is not negligible wrt to "
-            "the mean of all non-negative values."
-        )
-
-    array[negative_mask] = 0.0
