@@ -8,15 +8,20 @@ import numpy as np
 import pandas as pd
 import pandas.testing as pdt
 import pytest
-from voxcell import RegionMap  # type: ignore
+from voxcell import RegionMap
 
 import atlas_densities.densities.fitting as tested
-from atlas_densities.densities.utils import get_hierarchy_info
+from atlas_densities.densities import utils
 from atlas_densities.exceptions import AtlasDensitiesError, AtlasDensitiesWarning
 
 TESTS_PATH = Path(__file__).parent.parent
 
 
+@pytest.fixture
+def group_ids_config():
+    return utils.load_json(utils.GROUP_IDS_PATH)
+
+
 def test_create_dataframe_from_known_densities():
     average_densities = pd.DataFrame(
         {
@@ -44,7 +49,8 @@ def test_create_dataframe_from_known_densities():
     pdt.assert_frame_equal(actual, expected)
 
 
-def get_hierarchy_info_():
+@pytest.fixture
+def region_map():
     hierarchy = {
         "id": 8,
         "name": "Basic cell groups and regions",
@@ -98,12 +104,12 @@ def get_hierarchy_info_():
         ],
     }
 
-    return get_hierarchy_info(RegionMap.from_dict(hierarchy))
+    return RegionMap.from_dict(hierarchy)
 
 
 @pytest.fixture
-def hierarchy_info():
-    return get_hierarchy_info_()
+def hierarchy_info(region_map):
+    return utils.get_hierarchy_info(region_map)
 
 
 def test_fill_in_homogenous_regions(hierarchy_info):
@@ -219,9 +225,13 @@ def test_compute_average_intensity():
     assert np.allclose(actual, 0.3 / 2.0)
     actual = tested.compute_average_intensity(intensity, volume_mask, slices=[1])
     assert np.allclose(actual, 0.1 / 2.0)
+    actual = tested.compute_average_intensity(intensity, volume_mask, slices=[-1, 1, 3])
+    assert np.allclose(actual, 0.1 / 2.0)
+    actual = tested.compute_average_intensity(intensity, volume_mask, slices=[-1, 3])
+    assert actual == 0
 
 
-def test_compute_average_intensities(hierarchy_info):
+def test_compute_average_intensities(region_map, hierarchy_info):
     annotation = np.array(
         [[[0, 976], [976, 936]], [[976, 936], [936, 936]]]  # 976 = Lobule II, 936 = "Declive (VI)""
     )
@@ -256,32 +266,43 @@ def test_compute_average_intensities(hierarchy_info):
         index=hierarchy_info["brain_region"],
     )
 
-    actual = tested.compute_average_intensities(annotation, marker_volumes, hierarchy_info)
+    actual = tested.compute_average_intensities(
+        annotation, marker_volumes, hierarchy_info, region_map
+    )
     pdt.assert_frame_equal(actual, expected)
 
 
 def test_linear_fitting_xy():
-    actual = tested.linear_fitting_xy([0.0, 1.0, 2.0], [0.0, 2.0, 4.0], [1.0, 1.0, 1.0])
+    min_data_points = 1
+    actual = tested.linear_fitting_xy(
+        [0.0, 1.0, 2.0], [0.0, 2.0, 4.0], [1.0, 1.0, 1.0], min_data_points=min_data_points
+    )
     assert np.allclose(actual["coefficient"], 2.0)
+    assert np.allclose(actual["r_square"], 1.0)
     assert not np.isinf(actual["standard_deviation"])
 
-    actual = tested.linear_fitting_xy([0.0, 1.0, 2.0], [0.0, 1.0, 4.0], [1.0, 1.0, 1e-5])
+    actual = tested.linear_fitting_xy(
+        [0.0, 1.0, 2.0], [0.0, 1.0, 4.0], [1.0, 1.0, 1e-5], min_data_points=min_data_points
+    )
     assert np.allclose(actual["coefficient"], 2.0)
     assert not np.isinf(actual["standard_deviation"])
+    assert np.allclose(actual["r_square"], 0.89286)
 
-    actual = tested.linear_fitting_xy([0.0, 1.0, 2.0], [0.0, 2.0, 4.0], [1.0, 0.0, 1.0])
+    actual = tested.linear_fitting_xy(
+        [0.0, 1.0, 2.0], [0.0, 2.0, 4.0], [1.0, 0.0, 1.0], min_data_points=min_data_points
+    )
     assert np.allclose(actual["coefficient"], 2.0)
     assert not np.isinf(actual["standard_deviation"])
+    assert np.allclose(actual["r_square"], 1.0)
 
 
-def get_fitting_input_data_():
-    h = get_hierarchy_info_()
+def get_fitting_input_data_(hierarchy_info):
     intensities = pd.DataFrame(
         {
             "gad67": [7.0 / 4.0, 1.0, 2.0, 1.0, np.nan, np.nan, np.nan, np.nan],
             "pv": [10.0 / 7.0, 2.0, 1.0, 2.0, np.nan, np.nan, np.nan, np.nan],
         },
-        index=h["brain_region"],
+        index=hierarchy_info["brain_region"],
     )
     densities = pd.DataFrame(
         {
@@ -290,7 +311,7 @@ def get_fitting_input_data_():
             "pv+": [30.0 / 7.0, 6.0, 3.0, 6.0, np.nan, np.nan, np.nan, np.nan],
             "pv+_standard_deviation": [2.0, 1.0, 3.0, 4.0, np.nan, np.nan, np.nan, np.nan],
         },
-        index=h["brain_region"],
+        index=hierarchy_info["brain_region"],
     )
     data = {
         "groups": {"Whole": {"Lobule II", "Declive (VI)"}, "Central lobule": {"Lobule II"}},
@@ -301,11 +322,14 @@ def get_fitting_input_data_():
     return data
 
 
-def test_compute_fitting_coefficients():
-    data = get_fitting_input_data_()
+def test_compute_fitting_coefficients(hierarchy_info):
+    data = get_fitting_input_data_(hierarchy_info)
 
     actual = tested.compute_fitting_coefficients(
-        data["groups"], data["intensities"], data["densities"]
+        data["groups"],
+        data["intensities"],
+        data["densities"],
+        min_data_points=1,
     )
 
     for group_name in ["Whole", "Central lobule"]:
@@ -315,25 +339,44 @@ def test_compute_fitting_coefficients():
         assert not np.isnan(actual[group_name]["gad67+"]["standard_deviation"])
         assert not np.isinf(actual[group_name]["pv+"]["standard_deviation"])
         assert not np.isnan(actual[group_name]["pv+"]["standard_deviation"])
+    assert actual["Whole"]["pv+"]["r_square"] == 1.0
+    assert actual["Whole"]["gad67+"]["r_square"] == 1.0
+    assert np.isnan(actual["Central lobule"]["pv+"]["r_square"])  # only one value so no variation.
+    assert np.isnan(actual["Central lobule"]["gad67+"]["r_square"])
 
 
-def test_compute_fitting_coefficients_exceptions():
-    data = get_fitting_input_data_()
+def test_compute_fitting_coefficients_exceptions(hierarchy_info):
+    data = get_fitting_input_data_(hierarchy_info)
     data["densities"].drop(index=["Central lobule"], inplace=True)
 
     with pytest.raises(AtlasDensitiesError):
-        tested.compute_fitting_coefficients(data["groups"], data["intensities"], data["densities"])
+        tested.compute_fitting_coefficients(
+            data["groups"],
+            data["intensities"],
+            data["densities"],
+            min_data_points=1,
+        )
 
-    data = get_fitting_input_data_()
+    data = get_fitting_input_data_(hierarchy_info)
     data["densities"].drop(columns=["pv+"], inplace=True)
 
     with pytest.raises(AtlasDensitiesError):
-        tested.compute_fitting_coefficients(data["groups"], data["intensities"], data["densities"])
+        tested.compute_fitting_coefficients(
+            data["groups"],
+            data["intensities"],
+            data["densities"],
+            min_data_points=1,
+        )
 
-    data = get_fitting_input_data_()
+    data = get_fitting_input_data_(hierarchy_info)
     data["densities"].at["Lobule II", "pv+_standard_deviation"] = np.nan
     with pytest.raises(AssertionError):
-        tested.compute_fitting_coefficients(data["groups"], data["intensities"], data["densities"])
+        tested.compute_fitting_coefficients(
+            data["groups"],
+            data["intensities"],
+            data["densities"],
+            min_data_points=1,
+        )
 
 
 @pytest.fixture
@@ -419,6 +462,25 @@ def get_hierarchy():
                 "name": "Hippocampal formation",
                 "children": [],
             },
+            {
+                "id": 42,
+                "acronym": "PKL",
+                "name": "Purkinje layer",
+                "children": [],
+            },
+            {
+                "id": 38,
+                "acronym": "CCTX",
+                "name": "Cerebellar cortex",
+                "children": [
+                    {
+                        "id": 37,
+                        "acronym": "MLL",
+                        "name": "molecular layer",
+                        "children": [],
+                    },
+                ],
+            },
         ],
     }
 
@@ -476,7 +538,7 @@ def get_fitting_input_data():
     }
 
 
-def test_linear_fitting():
+def test_linear_fitting(group_ids_config):
     data = get_fitting_input_data()
 
     with warnings.catch_warnings(record=True) as warnings_:
@@ -487,10 +549,12 @@ def test_linear_fitting():
             data["gene_marker_volumes"],
             data["average_densities"],
             data["homogenous_regions"],
+            group_ids_config=group_ids_config,
+            min_data_points=1,
         )
         warnings_ = [w for w in warnings_ if isinstance(w.message, AtlasDensitiesWarning)]
         # Three warnings for recording NaN coefficients, three warnings for using them
-        assert len(warnings_) == 6
+        assert len(warnings_) == 9
 
     assert np.allclose(fitting_maps["Rest"]["pv+"]["coefficient"], 2.0 / 3.0)
 
@@ -503,7 +567,7 @@ def test_linear_fitting():
     assert not np.isinf(densities.at["Hippocampal formation", "pv+_standard_deviation"])
 
 
-def test_linear_fitting_exception_average_densities():
+def test_linear_fitting_exception_average_densities(group_ids_config):
     data = get_fitting_input_data()
     data["average_densities"].at["Thalamus", "measurement_type"] = "volume"
 
@@ -515,6 +579,7 @@ def test_linear_fitting_exception_average_densities():
             data["gene_marker_volumes"],
             data["average_densities"],
             data["homogenous_regions"],
+            group_ids_config=group_ids_config,
         )
 
     data["average_densities"].at["Thalamus", "measurement_type"] = "cell density"
@@ -528,10 +593,11 @@ def test_linear_fitting_exception_average_densities():
             data["gene_marker_volumes"],
             data["average_densities"],
             data["homogenous_regions"],
+            group_ids_config=group_ids_config,
         )
 
 
-def test_linear_fitting_exception_homogenous_regions():
+def test_linear_fitting_exception_homogenous_regions(group_ids_config):
     data = get_fitting_input_data()
     data["homogenous_regions"].at["Thalamus", "cell_type"] = "Inhibitory"
 
@@ -543,4 +609,12 @@ def test_linear_fitting_exception_homogenous_regions():
             data["gene_marker_volumes"],
             data["average_densities"],
             data["homogenous_regions"],
+            group_ids_config=group_ids_config,
         )
+
+
+def test__get_group_names(group_ids_config):
+    region_map = RegionMap.load_json(Path(TESTS_PATH, "1.json"))
+    ret = tested._get_group_names(region_map, group_ids_config=group_ids_config)
+    for k, length in (("Isocortex group", 409), ("Cerebellum group", 88), ("Rest", 825)):
+        assert length == len(ret[k])
