@@ -10,13 +10,13 @@ import pandas.testing as pdt
 import pytest
 
 import atlas_densities.densities.inhibitory_neuron_densities_optimization as tested
-from atlas_densities.exceptions import AtlasDensitiesError
+from atlas_densities.exceptions import AtlasDensitiesError, AtlasDensitiesWarning
 
 
 @pytest.fixture
 def counts_data_1():
     hierarchy_info = pd.DataFrame(
-        {"brain_region": ["A", "B"], "descendant_id_set": [{1}, {2}]}, index=[1, 2]
+        {"brain_region": ["A", "B"], "descendant_ids": [{1}, {2}]}, index=[1, 2]
     )
     region_counts = pd.DataFrame(
         {
@@ -162,7 +162,7 @@ def test_set_known_values_3(counts_data_1):
 def counts_data_2():
     hierarchy_info = pd.DataFrame(
         # A is now a parent region of B
-        {"brain_region": ["A", "B"], "descendant_id_set": [{1, 2}, {2}]},
+        {"brain_region": ["A", "B"], "descendant_ids": [{1, 2}, {2}]},
         index=[1, 2],
     )
     region_counts = pd.DataFrame(
@@ -335,7 +335,7 @@ def neuron_counts():
 
 def counts_data_3():
     hierarchy_info = pd.DataFrame(
-        {"brain_region": ["A", "B", "C"], "descendant_id_set": [{1, 2, 3}, {2}, {3}]},
+        {"brain_region": ["A", "B", "C"], "descendant_ids": [{1, 2, 3}, {2}, {3}]},
         index=[1, 2, 3],
     )
     region_counts = pd.DataFrame(
@@ -571,7 +571,7 @@ def test_create_bounds_2():
 @pytest.fixture
 def aub_and_bub_data():
     hierarchy_info = pd.DataFrame(
-        {"brain_region": ["A", "B", "C"], "descendant_id_set": [{1, 2, 3}, {2}, {3}]},
+        {"brain_region": ["A", "B", "C"], "descendant_ids": [{1, 2, 3}, {2}, {3}]},
         index=[1, 2, 3],
     )
     x_map, deltas_map = expected_maps()
@@ -961,13 +961,14 @@ def expected_aub_bub():
 
 
 def test_create_aub_and_bub(aub_and_bub_data):
-    A_ub, b_ub = tested.create_aub_and_bub(
-        aub_and_bub_data["x_result"],
-        aub_and_bub_data["region_counts"],
-        aub_and_bub_data["x_map"],
-        aub_and_bub_data["deltas_map"],
-        aub_and_bub_data["hierarchy_info"],
-    )
+    with pytest.warns(AtlasDensitiesWarning):
+        A_ub, b_ub = tested.create_aub_and_bub(
+            aub_and_bub_data["x_result"],
+            aub_and_bub_data["region_counts"],
+            aub_and_bub_data["x_map"],
+            aub_and_bub_data["deltas_map"],
+            aub_and_bub_data["hierarchy_info"],
+        )
 
     expected_A_ub, expected_b_ub = expected_aub_bub()
 
